/*
 * Copyright (c) 1999-2021 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_GEOMETRY_HPP
#define PW_CORE_GEOMETRY_HPP

#include <pw/core/globals.hpp>
#include <pw/core/vector.hpp>
#include <pw/core/aabb.hpp>
#include <pw/core/resource.hpp>

namespace pw {

/*
 * NOTE this needs to be rewritten to take into account for *any* kind of geometry
 * Some ideas are drafted down there to separate out the attribute buffers. Things to
 * consider: multiple UVs, triangle soup, per-vertex-color, texture transforms, weights,
 * etc. pp.
 */

class geometry {
public:

    enum class primitive_type {
        points,
        lines,
        triangles,
        polygons
    };

    using index_t = uint32_t;   //< needs to be compatible with GL_UNSIGNED_INT
    using indices_t = std::vector<index_t>;

    geometry() = default;
    geometry(primitive_type t, vector3_array v, indices_t i);

    void set_primitive_type(primitive_type t) { _primitive_type = t; }
    primitive_type primitive_type() { return _primitive_type; }

    void set_vertices(vector3_array v);
    const vector3_array& vertices() const { return _vertices; }

    void set_indices(indices_t v);
    const indices_t& indices() const;

    void set_normals(vector3_array v);
    const vector3_array& normals() const;

    void transform(const matrix4x4& m);

    void compute_normals();

    void compute_tangent_space();

    aabb bounds() const;

protected:

    enum primitive_type _primitive_type = primitive_type::triangles;

    vector3_array _vertices;   //!< vertex data
    indices_t _indices;        //!< indices

    vector3_array _normals;    //!< normal data
    vector3_array _tangents;   //!< tangent data
    vector3_array _bitangents; //!< bitangent

    std::vector<vector3_array> _texture_coords; //! texture coordinates

    uint64_t _change_count { 0 };
};

}

#endif

